@@ -2,10 +2,10 @@
 
 /* Marco preferences */
 
-/* 
+/*
  * Copyright (C) 2001 Havoc Pennington
  * Copyright (C) 2006 Elijah Newren
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -15,7 +15,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
@@ -130,7 +130,7 @@ gboolean    meta_prefs_get_force_fullscreen  (void);
  *
  * \param whether   TRUE to turn on, FALSE to turn off
  */
-void meta_prefs_set_compositing_manager (gboolean whether);
+void meta_prefs_set_force_compositing_manager (gboolean whether);
 
 void meta_prefs_set_compositing_fast_alt_tab (gboolean whether);
 
