@@ -41,28 +41,24 @@
 
 #if GTK_CHECK_VERSION(3, 0, 0)
     #include <cairo-xlib.h>
-    #define GtkObject GtkWidget
-    #define GTK_OBJECT_CLASS GTK_WIDGET_CLASS
     #define GdkRegion cairo_region_t
-    #define GdkRegion cairo_region_t
-    #define gdk_region_new cairo_region_create
-    #define gdk_region_subtract cairo_region_subtract
     #define gdk_region_destroy cairo_region_destroy
-    #define gdk_region_union_with_rect cairo_region_union_rectangle
     #define gdk_region_rectangle cairo_region_create_rectangle
     #define gdk_region_offset cairo_region_translate
     #define gdk_region_intersect cairo_region_intersect
-    #define gdk_region_copy cairo_region_copy
-    G_DEFINE_TYPE (MetaFrames, meta_frames, GTK_TYPE_WINDOW);
+    G_DEFINE_TYPE (MetaFrames, meta_frames, GTK_TYPE_INVISIBLE);
     #define parent_class meta_frames_parent_class
-    #define GTK_WIDGET_REALIZED gtk_widget_get_realized
 #endif
 
 #define DEFAULT_INNER_BUTTON_BORDER 3
 
 static void meta_frames_class_init (MetaFramesClass *klass);
 static void meta_frames_init       (MetaFrames      *frames);
+#if GTK_CHECK_VERSION(3, 0, 0)
+static void meta_frames_destroy    (GtkWidget       *object);
+#else
 static void meta_frames_destroy    (GtkObject       *object);
+#endif
 static void meta_frames_finalize   (GObject         *object);
 static void meta_frames_style_set  (GtkWidget       *widget,
                                     GtkStyle        *prev_style);
@@ -134,27 +130,7 @@ static void invalidate_all_caches (MetaFrames *frames);
 static void invalidate_whole_window (MetaFrames *frames,
                                      MetaUIFrame *frame);
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-
-static GObject *
-meta_frames_constructor (GType                  gtype,
-                         guint                  n_properties,
-                         GObjectConstructParam *properties)
-{
-  GObject *object;
-  GObjectClass *gobject_class;
-
-  gobject_class = G_OBJECT_CLASS (meta_frames_parent_class);
-  object = gobject_class->constructor (gtype, n_properties, properties);
-
-  g_object_set (object,
-                "type", GTK_WINDOW_POPUP,
-                NULL);
-
-  return object;
-}
-
-#else
+#if !GTK_CHECK_VERSION(3, 0, 0)
 
 static GtkWidgetClass *parent_class = NULL;
 
@@ -200,11 +176,15 @@ meta_frames_class_init (MetaFramesClass *class)
   #endif
   widget_class = (GtkWidgetClass*) class;
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
   parent_class = g_type_class_peek_parent (class);
+#endif
 
   gobject_class->finalize = meta_frames_finalize;
   #if !GTK_CHECK_VERSION(3, 0, 0)
   object_class->destroy = meta_frames_destroy;
+  #else
+  widget_class->destroy = meta_frames_destroy;
   #endif
 
   widget_class->style_set = meta_frames_style_set;
@@ -295,8 +275,13 @@ listify_func (gpointer key, gpointer value, gpointer data)
   *listp = g_slist_prepend (*listp, value);
 }
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+static void
+meta_frames_destroy (GtkWidget *object)
+#else
 static void
 meta_frames_destroy (GtkObject *object)
+#endif
 {
   GSList *winlist;
   GSList *tmp;
@@ -320,7 +305,11 @@ meta_frames_destroy (GtkObject *object)
     }
   g_slist_free (winlist);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  GTK_WIDGET_CLASS (parent_class)->destroy (object);
+#else
   GTK_OBJECT_CLASS (parent_class)->destroy (object);
+#endif
 }
 
 static void
@@ -533,7 +522,7 @@ meta_frames_ensure_layout (MetaFrames  *frames,
   MetaFrameType type;
   MetaFrameStyle *style;
 
-  g_return_if_fail (GTK_WIDGET_REALIZED (GTK_WIDGET(frames)));
+  g_return_if_fail (gtk_widget_get_realized (GTK_WIDGET(frames)));
 
   widget = GTK_WIDGET (frames);
 
@@ -663,16 +652,19 @@ meta_frames_attach_style (MetaFrames  *frames,
                           MetaUIFrame *frame)
 {
   if (frame->style != NULL)
+#if GTK_CHECK_VERSION(3, 0, 0)
+    g_object_unref (frame->style);
+#else
     gtk_style_detach (frame->style);
+#endif
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  frame->style = g_object_ref (gtk_widget_get_style_context (GTK_WIDGET (frames)));
+#else
   /* Weirdly, gtk_style_attach() steals a reference count from the style passed in */
-  #if GTK_CHECK_VERSION(3, 0, 0)
-  g_object_ref (gtk_widget_get_style (GTK_WIDGET (frames)));
-  frame->style = gtk_style_attach (gtk_widget_get_style (GTK_WIDGET (frames)), frame->window);
-  #else
   g_object_ref (GTK_WIDGET (frames)->style);
   frame->style = gtk_style_attach (GTK_WIDGET (frames)->style, frame->window);
-  #endif
+#endif
 }
 
 void
@@ -743,7 +735,11 @@ meta_frames_unmanage_window (MetaFrames *frames,
 
       g_hash_table_remove (frames->frames, &frame->xwindow);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+      g_object_unref (frame->style);
+#else
       gtk_style_detach (frame->style);
+#endif
 
       gdk_window_destroy (frame->window);
 
@@ -1103,16 +1099,14 @@ meta_frames_move_resize_frame (MetaFrames *frames,
                                int         height)
 {
   MetaUIFrame *frame = meta_frames_lookup_window (frames, xwindow);
-  int old_x, old_y, old_width, old_height;
+  int old_width, old_height;
 
-	#if GTK_CHECK_VERSION(3, 0, 0)
-		old_width = gdk_window_get_width(GDK_WINDOW(frame->window));
-		old_height = gdk_window_get_height(GDK_WINDOW(frame->window));
-	#else
-		gdk_drawable_get_size(frame->window, &old_width, &old_height);
-	#endif
-
-  gdk_window_get_position (frame->window, &old_x, &old_y);
+#if GTK_CHECK_VERSION(3, 0, 0)
+  old_width = gdk_window_get_width (frame->window);
+  old_height = gdk_window_get_height (frame->window);
+#else
+  gdk_drawable_get_size(frame->window, &old_width, &old_height);
+#endif
 
   gdk_window_move_resize (frame->window, x, y, width, height);
 
@@ -1274,8 +1268,7 @@ show_tip_now (MetaFrames *frames)
 
       screen_number = gdk_screen_get_number (gtk_widget_get_screen (GTK_WIDGET (frames)));
 
-      meta_fixed_tip_show (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-                           screen_number,
+      meta_fixed_tip_show (screen_number,
                            rect->x + dx,
                            rect->y + rect->height + 2 + dy,
                            tiptext);
@@ -1291,6 +1284,8 @@ tip_timeout_func (gpointer data)
 
   show_tip_now (frames);
 
+  frames->tooltip_timeout = 0;
+
   return FALSE;
 }
 
@@ -2029,7 +2024,12 @@ meta_frames_motion_notify_event     (GtkWidget           *widget,
         MetaFrameControl control;
         int x, y;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+        gdk_window_get_device_position (frame->window, event->device,
+                                        &x, &y, NULL);
+#else
         gdk_window_get_pointer (frame->window, &x, &y, NULL);
+#endif
 
         /* Control is set to none unless it matches
          * the current grab
@@ -2074,7 +2074,12 @@ meta_frames_motion_notify_event     (GtkWidget           *widget,
         MetaFrameControl control;
         int x, y;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+        gdk_window_get_device_position (frame->window, event->device,
+                                        &x, &y, NULL);
+#else
         gdk_window_get_pointer (frame->window, &x, &y, NULL);
+#endif
 
         control = get_control (frames, frame, x, y);
 
@@ -2108,54 +2113,6 @@ meta_frames_destroy_event           (GtkWidget           *widget,
   return TRUE;
 }
 
-#if !GTK_CHECK_VERSION(2,21,6)
-/* Copied from GDK */
-static cairo_surface_t *
-_gdk_drawable_ref_cairo_surface (GdkDrawable *drawable)
-{
-  g_return_val_if_fail (GDK_IS_DRAWABLE (drawable), NULL);
-
-  return GDK_DRAWABLE_GET_CLASS (drawable)->ref_cairo_surface (drawable);
-}
-
-static cairo_pattern_t *
-gdk_window_get_background_pattern (GdkWindow *window)
-{
-  GdkWindowObject *private = (GdkWindowObject *) window;
-  cairo_pattern_t *pattern;
-
-  g_return_val_if_fail (GDK_IS_WINDOW (window), NULL);
-
-  if (private->bg_pixmap == GDK_PARENT_RELATIVE_BG)
-    pattern = NULL;
-  else if (private->bg_pixmap != GDK_NO_BG &&
-           private->bg_pixmap != NULL)
-    {
-      static cairo_user_data_key_t key;
-      cairo_surface_t *surface;
-
-      surface = _gdk_drawable_ref_cairo_surface (private->bg_pixmap);
-      pattern = cairo_pattern_create_for_surface (surface);
-      cairo_surface_destroy (surface);
-
-      cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
-      cairo_pattern_set_user_data (pattern,
-                                   &key,
-                                   g_object_ref (private->bg_pixmap),
-                                   g_object_unref);
-    }
-  else
-    {
-      pattern =
-        cairo_pattern_create_rgb (private->bg_color.red   / 65535.,
-                                  private->bg_color.green / 65535.,
-                                  private->bg_color.blue / 65535.);
-    }
-
-  return pattern;
-}
-#endif
-
 static void
 setup_bg_cr (cairo_t *cr, GdkWindow *window, int x_offset, int y_offset)
 {
@@ -2222,7 +2179,7 @@ generate_pixmap (MetaFrames            *frames,
   result = gdk_window_create_similar_surface (frame->window,
                                               CAIRO_CONTENT_COLOR,
                                               rect->width, rect->height);
-  
+
   cr = cairo_create (result);
   cairo_translate (cr, -rect->x, -rect->y);
 
@@ -2420,13 +2377,13 @@ cached_pixels_draw (CachedPixels   *pixels,
     {
       CachedFramePiece *piece;
       piece = &pixels->piece[i];
-      
+
       if (piece->pixmap)
         {
           cairo_set_source_surface (cr, piece->pixmap,
                                     piece->rect.x, piece->rect.y);
           cairo_paint (cr);
-          
+
           region_piece = cairo_region_create_rectangle (&piece->rect);
           cairo_region_subtract (region, region_piece);
           cairo_region_destroy (region_piece);
@@ -2476,7 +2433,7 @@ subtract_client_area (cairo_region_t *region, MetaUIFrame *frame)
   MetaFrameType type;
   cairo_region_t *tmp_region;
   Display *display;
-  
+
   display = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
 
   meta_core_get (display, frame->xwindow,
@@ -2486,7 +2443,7 @@ subtract_client_area (cairo_region_t *region, MetaUIFrame *frame)
                  META_CORE_GET_CLIENT_HEIGHT, &area.height,
                  META_CORE_GET_END);
   meta_theme_get_frame_borders (meta_theme_get_current (),
-                         type, frame->text_height, flags, 
+                         type, frame->text_height, flags,
                          &area.x, NULL, &area.y, NULL);
 
   tmp_region = cairo_region_create_rectangle (&area);
@@ -2525,11 +2482,11 @@ meta_frames_draw (GtkWidget *widget,
   populate_cache (frames, frame);
 
   region = cairo_region_create_rectangle (&clip);
-  
+
   pixels = get_cache (frames, frame);
 
   cached_pixels_draw (pixels, cr, region);
-  
+
   clip_to_screen (region, frame);
   subtract_client_area (region, frame);
 
@@ -2557,7 +2514,7 @@ meta_frames_draw (GtkWidget *widget,
     }
 
   cairo_region_destroy (region);
-  
+
   return TRUE;
 }
 
@@ -2620,7 +2577,6 @@ meta_frames_paint_to_drawable (MetaFrames   *frames,
                                int           y_offset)
                                #endif
 {
-  GtkWidget *widget;
   MetaFrameFlags flags;
   MetaFrameType type;
   GdkPixbuf *mini_icon;
@@ -2632,8 +2588,6 @@ meta_frames_paint_to_drawable (MetaFrames   *frames,
   MetaButtonLayout button_layout;
   MetaGrabOp grab_op;
 
-  widget = GTK_WIDGET (frames);
-
   for (i = 0; i < META_BUTTON_TYPE_LAST; i++)
     button_states[i] = META_BUTTON_STATE_NORMAL;
 
@@ -2717,20 +2671,6 @@ meta_frames_paint_to_drawable (MetaFrames   *frames,
       break;
     }
 
-  /* Map button function states to button position states */
-  button_states[META_BUTTON_TYPE_LEFT_LEFT_BACKGROUND] =
-    button_states[META_BUTTON_TYPE_MENU];
-  button_states[META_BUTTON_TYPE_LEFT_MIDDLE_BACKGROUND] =
-    META_BUTTON_STATE_NORMAL;
-  button_states[META_BUTTON_TYPE_LEFT_RIGHT_BACKGROUND] =
-    META_BUTTON_STATE_NORMAL;
-  button_states[META_BUTTON_TYPE_RIGHT_LEFT_BACKGROUND] =
-    button_states[META_BUTTON_TYPE_MINIMIZE];
-  button_states[META_BUTTON_TYPE_RIGHT_MIDDLE_BACKGROUND] =
-    button_states[META_BUTTON_TYPE_MAXIMIZE];
-  button_states[META_BUTTON_TYPE_RIGHT_RIGHT_BACKGROUND] =
-    button_states[META_BUTTON_TYPE_CLOSE];
-
   meta_core_get (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), frame->xwindow,
                  META_CORE_GET_FRAME_FLAGS, &flags,
                  META_CORE_GET_FRAME_TYPE, &type,
@@ -2748,7 +2688,6 @@ meta_frames_paint_to_drawable (MetaFrames   *frames,
 
   meta_theme_draw_frame_with_style (meta_theme_get_current (),
                                     frame->style,
-                                    widget,
                                     cr,
                                     type,
                                     flags,
@@ -2821,8 +2760,11 @@ meta_frames_paint_to_drawable (MetaFrames   *frames,
           gdk_window_begin_paint_rect (drawable, &areas[i]);
 
           meta_theme_draw_frame_with_style (meta_theme_get_current (),
+#if GTK_CHECK_VERSION (3, 0, 0)
             frame->style,
-            widget,
+#else
+            GTK_WIDGET(frames),
+#endif
             drawable,
             NULL, /* &areas[i], */
             x_offset, y_offset,
@@ -2847,8 +2789,11 @@ meta_frames_paint_to_drawable (MetaFrames   *frames,
       /* Not a window; happens about 1/3 of the time */
 
       meta_theme_draw_frame_with_style (meta_theme_get_current (),
+#if GTK_CHECK_VERSION (3, 0, 0)
                                         frame->style,
-                                        widget,
+#else
+                                        GTK_WIDGET(frames),
+#endif
                                         drawable,
                                         NULL,
                                         x_offset, y_offset,
@@ -2889,32 +2834,44 @@ meta_frames_set_window_background (MetaFrames   *frames,
 
   if (frame_exists && style->window_background_color != NULL)
     {
+#if GTK_CHECK_VERSION (3, 0, 0)
+      GdkRGBA color;
+#else
       GdkColor color;
+#endif
       GdkVisual *visual;
 
       meta_color_spec_render (style->window_background_color,
-                              GTK_WIDGET (frames),
+                              frame->style,
                               &color);
 
       /* Set A in ARGB to window_background_alpha, if we have ARGB */
 
       visual = gtk_widget_get_visual (GTK_WIDGET (frames));
+      if (gdk_visual_get_depth (visual) == 32) /* we have ARGB */
       #if GTK_CHECK_VERSION(3, 0, 0)
-      if (gdk_visual_get_depth(visual) == 32) /* we have ARGB */
+        {
+          color.alpha = style->window_background_alpha / 255.0;
+        }
+
+      gdk_window_set_background_rgba (frame->window, &color);
       #else
-      if (visual->depth == 32) /* we have ARGB */
-      #endif
         {
           color.pixel = (color.pixel & 0xffffff) &
             style->window_background_alpha << 24;
         }
 
       gdk_window_set_background (frame->window, &color);
+      #endif
     }
   else
     {
+#if GTK_CHECK_VERSION (3, 0, 0)
+      gtk_style_context_set_background (frame->style, frame->window);
+#else
       gtk_style_set_background (frame->style,
                                 frame->window, GTK_STATE_NORMAL);
+#endif
     }
  }
 
