@@ -28,7 +28,6 @@
 #include "main.h"
 #include "util.h"
 #include "core.h"
-#include "themewidget.h"
 #include "metaaccellabel.h"
 #include "ui.h"
 
@@ -107,7 +106,11 @@ static void popup_position_func(GtkMenu* menu, gint* x, gint* y, gboolean* push_
 
 	pos = user_data;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &req, NULL);
+#else
 	gtk_widget_size_request(GTK_WIDGET(menu), &req);
+#endif
 
 	*x = pos->x;
 	*y = pos->y;
@@ -149,7 +152,7 @@ static void activate_cb(GtkWidget* menuitem, gpointer data)
 
   md = data;
 
-	meta_frames_notify_menu_hide(md->menu->frames);
+  meta_frames_notify_menu_hide (md->menu->frames);
 
 	(*md->menu->func)(
 		md->menu,
@@ -160,7 +163,7 @@ static void activate_cb(GtkWidget* menuitem, gpointer data)
 		GPOINTER_TO_INT(g_object_get_data(G_OBJECT(menuitem), "workspace")),
 		md->menu->data);
 
-	/* menu may now be freed */
+  /* menu may now be freed */
 }
 
 /*
@@ -390,19 +393,12 @@ meta_window_menu_new   (MetaFrames         *frames,
                   meta_verbose ("Creating %d-workspace menu current space %lu\n",
                       n_workspaces, active_workspace);
 
-                  #if GTK_CHECK_VERSION(3, 0, 0)
-                  GdkWindow* window = gtk_widget_get_window(GTK_WIDGET(frames));
+                  GdkWindow* window = gtk_widget_get_window (GTK_WIDGET (frames));
 
                   display = GDK_WINDOW_XDISPLAY (window);
 
                   screen = gdk_window_get_screen (window);
                   xroot = GDK_WINDOW_XID (gdk_screen_get_root_window (screen));
-                  #else
-                  display = gdk_x11_drawable_get_xdisplay (GTK_WIDGET (frames)->window);
-
-                  screen = gdk_drawable_get_screen (GTK_WIDGET (frames)->window);
-                  xroot = GDK_DRAWABLE_XID (gdk_screen_get_root_window (screen));
-                  #endif
 
                   submenu = gtk_menu_new ();
 
@@ -505,9 +501,10 @@ meta_window_menu_new   (MetaFrames         *frames,
         }
     }
 
-	g_signal_connect (menu->menu, "selection_done", G_CALLBACK(menu_closed), menu);
+  g_signal_connect (menu->menu, "selection_done",
+                    G_CALLBACK (menu_closed), menu);
 
-	return menu;
+  return menu;
 }
 
 void meta_window_menu_popup(MetaWindowMenu* menu, int root_x, int root_y, int button, guint32 timestamp)
