@@ -2,11 +2,11 @@
 
 /* Marco popup window thing showing windows you can tab to */
 
-/* 
+/*
  * Copyright (C) 2001 Havoc Pennington
  * Copyright (C) 2002 Red Hat, Inc.
  * Copyright (C) 2005 Elijah Newren
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -16,7 +16,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
@@ -37,11 +37,12 @@
 #include <math.h>
 
 #if GTK_CHECK_VERSION(3, 0, 0)
-    #define GTK_WIDGET_VISIBLE gtk_widget_get_visible
     #define GdkRegion cairo_region_t
     #define gdk_region_subtract cairo_region_subtract
     #define gdk_region_destroy cairo_region_destroy
     #define gdk_region_rectangle cairo_region_create_rectangle
+    /* Silence Gtk{V,H}Box deprecations warnings */
+    #define gtk_vbox_new(X, Y) gtk_box_new(GTK_ORIENTATION_VERTICAL, Y)
 #endif
 
 #define OUTSIDE_SELECT_RECT 2
@@ -80,28 +81,42 @@ static void       select_workspace         (GtkWidget *widget);
 static void       unselect_workspace       (GtkWidget *widget);
 
 static gboolean
+#if GTK_CHECK_VERSION (3, 0, 0)
+outline_window_draw (GtkWidget *widget,
+                     cairo_t   *cr,
+                     gpointer   data)
+#else
 outline_window_expose (GtkWidget      *widget,
                        GdkEventExpose *event,
                        gpointer        data)
+#endif
 {
   MetaTabPopup *popup;
-  TabEntry *te;  
+  TabEntry *te;
+#if !GTK_CHECK_VERSION (3, 0, 0)
   GtkStyle *style;
   GdkWindow *window;
   cairo_t *cr;
-  
+#endif
+
   popup = data;
 
   if (!popup->outline || popup->current_selected_entry == NULL)
     return FALSE;
 
   te = popup->current_selected_entry;
+#if !GTK_CHECK_VERSION (3, 0, 0)
   window = gtk_widget_get_window (widget);
   style = gtk_widget_get_style (widget);
   cr = gdk_cairo_create (window);
-  
+#endif
+
   cairo_set_line_width (cr, 1.0);
+#if GTK_CHECK_VERSION (3, 0, 0)
+  cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
+#else
   gdk_cairo_set_source_color (cr, &style->white);
+#endif
 
   cairo_rectangle (cr,
                    0.5, 0.5,
@@ -115,7 +130,9 @@ outline_window_expose (GtkWidget      *widget,
                    te->inner_rect.height + 1);
   cairo_stroke (cr);
 
+#if !GTK_CHECK_VERSION (3, 0, 0)
   cairo_destroy (cr);
+#endif
 
   return FALSE;
 }
@@ -138,7 +155,7 @@ dimm_icon (GdkPixbuf *pixbuf)
     }
 
   w = gdk_pixbuf_get_width (dimmed_pixbuf);
-  h = gdk_pixbuf_get_height (dimmed_pixbuf);      
+  h = gdk_pixbuf_get_height (dimmed_pixbuf);
 
   pixel_stride = 4;
 
@@ -147,24 +164,24 @@ dimm_icon (GdkPixbuf *pixbuf)
 
   for (y = 0; y < h; y++)
     {
-      pixels = row;                     
-      for (x = 0; x < w; x++) 
+      pixels = row;
+      for (x = 0; x < w; x++)
         {
-          pixels[3] /= 2;                               
+          pixels[3] /= 2;
           pixels += pixel_stride;
-        }                       
+        }
       row += row_stride;
     }
   return dimmed_pixbuf;
 }
 
-static TabEntry*  
-tab_entry_new (const MetaTabEntry *entry, 
+static TabEntry*
+tab_entry_new (const MetaTabEntry *entry,
                gint                screen_width,
                gboolean            outline)
 {
   TabEntry *te;
-  
+
   te = g_new (TabEntry, 1);
   te->key = entry->key;
   te->title = NULL;
@@ -185,9 +202,9 @@ tab_entry_new (const MetaTabEntry *entry,
       g_free (str);
       str = tmp;
 
-      if (entry->demands_attention) 
-        {         
-          /* Escape the whole line of text then markup the text and 
+      if (entry->demands_attention)
+        {
+          /* Escape the whole line of text then markup the text and
            * copy it back into the original buffer.
            */
           tmp = g_strdup_printf ("<b>%s</b>", str);
@@ -209,7 +226,7 @@ tab_entry_new (const MetaTabEntry *entry,
       if (entry->hidden)
         te->dimmed_icon = dimm_icon (entry->icon);
     }
-  
+
   if (outline)
     {
       te->rect.x = entry->rect.x;
@@ -235,7 +252,11 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   MetaTabPopup *popup;
   int i, left, right, top, bottom;
   int height;
+#if GTK_CHECK_VERSION (3, 0, 0)
+  GtkWidget *grid;
+#else
   GtkWidget *table;
+#endif
   GtkWidget *vbox;
   GtkWidget *align;
   GList *tmp;
@@ -244,7 +265,7 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   AtkObject *obj;
   GdkScreen *screen;
   int screen_width;
-  
+
   popup = g_new (MetaTabPopup, 1);
 
   popup->outline_window = gtk_window_new (GTK_WINDOW_POPUP);
@@ -257,9 +278,14 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   gtk_widget_set_app_paintable (popup->outline_window, TRUE);
   gtk_widget_realize (popup->outline_window);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+  g_signal_connect (G_OBJECT (popup->outline_window), "draw",
+                    G_CALLBACK (outline_window_draw), popup);
+#else
   g_signal_connect (G_OBJECT (popup->outline_window), "expose_event",
                     G_CALLBACK (outline_window_expose), popup);
-  
+#endif
+
   popup->window = gtk_window_new (GTK_WINDOW_POPUP);
 
   gtk_window_set_screen (GTK_WINDOW (popup->window),
@@ -283,18 +309,27 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
     }
 
   popup->entries = g_list_reverse (popup->entries);
-    
+
   g_assert (width > 0);
   height = i / width;
   if (i % width)
     height += 1;
 
-  table = gtk_table_new (height, width, FALSE);
   vbox = gtk_vbox_new (FALSE, 0);
-  
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+  grid = gtk_grid_new ();
+#else
+  table = gtk_table_new (height, width, FALSE);
+#endif
+
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
+#if GTK_CHECK_VERSION (3, 0, 0)
+  gtk_container_set_border_width (GTK_CONTAINER (grid), 1);
+#else
   gtk_container_set_border_width (GTK_CONTAINER (table), 1);
+#endif
   gtk_container_add (GTK_CONTAINER (popup->window),
                      frame);
   gtk_container_add (GTK_CONTAINER (frame),
@@ -304,8 +339,13 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
 
   gtk_box_pack_start (GTK_BOX (vbox), align, TRUE, TRUE, 0);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+  gtk_container_add (GTK_CONTAINER (align),
+                     grid);
+#else
   gtk_container_add (GTK_CONTAINER (align),
                      table);
+#endif
 
   popup->label = gtk_label_new ("");
 
@@ -326,7 +366,7 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   tmp = popup->entries;
 
   while (tmp && top < height)
-    {      
+    {
       left = 0;
       right = 1;
 
@@ -350,7 +390,7 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
                 {
                   image = selectable_image_new (te->dimmed_icon);
                 }
-              else 
+              else
                 {
                   image = selectable_image_new (te->icon);
                 }
@@ -359,7 +399,7 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
                                     INSIDE_SELECT_RECT + OUTSIDE_SELECT_RECT + 1,
                                     INSIDE_SELECT_RECT + OUTSIDE_SELECT_RECT + 1);
               gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.5);
-            }   
+            }
           else
             {
               image = selectable_workspace_new ((MetaWorkspace *) te->key);
@@ -367,24 +407,32 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
 
           te->widget = image;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+          gtk_grid_attach (GTK_GRID (grid), te->widget, left, top, 1, 1);
+#else
           gtk_table_attach (GTK_TABLE (table),
                             te->widget,
                             left, right,           top, bottom,
                             0,                     0,
                             0,                     0);
+#endif
 
           /* Efficiency rules! */
           gtk_label_set_markup (GTK_LABEL (popup->label),
                               te->title);
+#if GTK_CHECK_VERSION (3, 0, 0)
+          gtk_widget_get_preferred_size (popup->label, &req, NULL);
+#else
           gtk_widget_size_request (popup->label, &req);
+#endif
           max_label_width = MAX (max_label_width, req.width);
-          
+
           tmp = tmp->next;
-          
+
           ++left;
           ++right;
         }
-      
+
       ++top;
       ++bottom;
     }
@@ -395,17 +443,17 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   gtk_label_set_ellipsize (GTK_LABEL (popup->label), PANGO_ELLIPSIZE_END);
 
   /* Limit the window size to no bigger than screen_width/4 */
-  if (max_label_width>(screen_width/4)) 
+  if (max_label_width>(screen_width/4))
     {
       max_label_width = screen_width/4;
     }
 
   max_label_width += 20; /* add random padding */
-  
+
   gtk_window_set_default_size (GTK_WINDOW (popup->window),
                                max_label_width,
                                -1);
-  
+
   return popup;
 }
 
@@ -415,7 +463,7 @@ free_tab_entry (gpointer data, gpointer user_data)
   TabEntry *te;
 
   te = data;
-  
+
   g_free (te->title);
   if (te->icon)
     g_object_unref (G_OBJECT (te->icon));
@@ -429,14 +477,15 @@ void
 meta_ui_tab_popup_free (MetaTabPopup *popup)
 {
   meta_verbose ("Destroying tab popup window\n");
-  
-  gtk_widget_destroy (popup->outline_window);
+
+  if (popup->outline_window != NULL)
+    gtk_widget_destroy (popup->outline_window);
   gtk_widget_destroy (popup->window);
-  
+
   g_list_foreach (popup->entries, free_tab_entry, NULL);
 
   g_list_free (popup->entries);
-  
+
   g_free (popup);
 }
 
@@ -450,7 +499,7 @@ meta_ui_tab_popup_set_showing (MetaTabPopup *popup,
     }
   else
     {
-      if (GTK_WIDGET_VISIBLE (popup->window))
+      if (gtk_widget_get_visible (popup->window))
         {
           meta_verbose ("Hiding tab popup window\n");
           gtk_widget_hide (popup->window);
@@ -468,7 +517,7 @@ display_entry (MetaTabPopup *popup,
   GdkRegion *inner_region;
   GdkWindow *window;
 
-  
+
   if (popup->current_selected_entry)
   {
     if (popup->outline)
@@ -476,7 +525,7 @@ display_entry (MetaTabPopup *popup,
     else
       unselect_workspace (popup->current_selected_entry->widget);
   }
-  
+
   gtk_label_set_markup (GTK_LABEL (popup->label), te->title);
 
   if (popup->outline)
@@ -489,7 +538,7 @@ display_entry (MetaTabPopup *popup,
       /* Do stuff behind gtk's back */
       gdk_window_hide (gtk_widget_get_window(popup->outline_window));
       meta_core_increment_event_serial (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
-  
+
       rect = te->rect;
       rect.x = 0;
       rect.y = 0;
@@ -499,7 +548,7 @@ display_entry (MetaTabPopup *popup,
       gdk_window_move_resize (window,
                               te->rect.x, te->rect.y,
                               te->rect.width, te->rect.height);
-  
+
       #if GTK_CHECK_VERSION(3, 0, 0)
       GdkRGBA black = { 0.0, 0.0, 0.0, 1.0 };
       gdk_window_set_background_rgba (window, &black);
@@ -507,28 +556,24 @@ display_entry (MetaTabPopup *popup,
       gdk_window_set_background (window,
                                  &popup->outline_window->style->black);
       #endif
-  
+
       region = gdk_region_rectangle (&rect);
       inner_region = gdk_region_rectangle (&te->inner_rect);
       gdk_region_subtract (region, inner_region);
       gdk_region_destroy (inner_region);
-  
+
       gdk_window_shape_combine_region (window,
                                        region,
                                        0, 0);
 
       gdk_region_destroy (region);
-  
+
       /* This should piss off gtk a bit, but we don't want to raise
        * above the tab popup.  So, instead of calling gtk_widget_show,
        * we manually set the window as mapped and then manually map it
        * with gdk functions.
        */
-      #if GTK_CHECK_VERSION(3, 0, 0)
       gtk_widget_set_mapped (popup->outline_window, TRUE);
-      #else
-      GTK_WIDGET_SET_FLAGS (popup->outline_window, GTK_MAPPED);
-      #endif
       gdk_window_show_unraised (window);
     }
 
@@ -544,7 +589,7 @@ meta_ui_tab_popup_forward (MetaTabPopup *popup)
 
   if (popup->current == NULL)
     popup->current = popup->entries;
-  
+
   if (popup->current != NULL)
     {
       TabEntry *te;
@@ -563,7 +608,7 @@ meta_ui_tab_popup_backward (MetaTabPopup *popup)
 
   if (popup->current == NULL)
     popup->current = g_list_last (popup->entries);
-  
+
   if (popup->current != NULL)
     {
       TabEntry *te;
@@ -598,7 +643,7 @@ meta_ui_tab_popup_select (MetaTabPopup *popup,
   /* Note, "key" may not be in the list of entries; other code assumes
    * it's OK to pass in a key that isn't.
    */
-  
+
   tmp = popup->entries;
   while (tmp != NULL)
     {
@@ -609,12 +654,12 @@ meta_ui_tab_popup_select (MetaTabPopup *popup,
       if (te->key == key)
         {
           popup->current = tmp;
-          
+
           display_entry (popup, te);
 
           return;
         }
-      
+
       tmp = tmp->next;
     }
 }
@@ -645,7 +690,7 @@ selectable_image_new (GdkPixbuf *pixbuf)
   GtkWidget *w;
 
   w = g_object_new (meta_select_image_get_type (), NULL);
-  gtk_image_set_from_pixbuf (GTK_IMAGE (w), pixbuf); 
+  gtk_image_set_from_pixbuf (GTK_IMAGE (w), pixbuf);
 
   return w;
 }
@@ -705,14 +750,14 @@ static void
 meta_select_image_class_init (MetaSelectImageClass *klass)
 {
   GtkWidgetClass *widget_class;
-  
+
   parent_class = g_type_class_peek (gtk_image_get_type ());
 
   widget_class = GTK_WIDGET_CLASS (klass);
 
   #if GTK_CHECK_VERSION(3, 0, 0)
   widget_class->draw = meta_select_image_draw;
-  #else  
+  #else
   widget_class->expose_event = meta_select_image_expose_event;
   #endif
 }
@@ -739,16 +784,20 @@ meta_select_image_draw (GtkWidget *widget,
 
       misc = GTK_MISC (widget);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+      gtk_widget_get_preferred_size (widget, &requisition, NULL);
+#else
       gtk_widget_get_requisition (widget, &requisition);
+#endif
       gtk_misc_get_alignment (misc, &xalign, &yalign);
       gtk_misc_get_padding (misc, &xpad, &ypad);
-      
+
       x = (allocation.width - (requisition.width - xpad * 2)) * xalign + 0.5;
       y = (allocation.height - (requisition.height - ypad * 2)) * yalign + 0.5;
 
       x -= INSIDE_SELECT_RECT + 1;
-      y -= INSIDE_SELECT_RECT + 1;      
-      
+      y -= INSIDE_SELECT_RECT + 1;
+
       w = requisition.width - OUTSIDE_SELECT_RECT * 2 - 1;
       h = requisition.height - OUTSIDE_SELECT_RECT * 2 - 1;
 
@@ -789,7 +838,7 @@ meta_select_image_expose_event (GtkWidget      *widget,
       gtk_widget_get_allocation(widget, &allocation);
 
       misc = GTK_MISC (widget);
-      
+
       x = (allocation.x * (1.0 - misc->xalign) +
            (allocation.x + allocation.width
             - (widget->requisition.width - misc->xpad * 2)) *
@@ -800,8 +849,8 @@ meta_select_image_expose_event (GtkWidget      *widget,
            misc->yalign) + 0.5;
 
       x -= INSIDE_SELECT_RECT + 1;
-      y -= INSIDE_SELECT_RECT + 1;      
-      
+      y -= INSIDE_SELECT_RECT + 1;
+
       w = widget->requisition.width - OUTSIDE_SELECT_RECT * 2 - 1;
       h = widget->requisition.height - OUTSIDE_SELECT_RECT * 2 - 1;
 
@@ -822,7 +871,7 @@ meta_select_image_expose_event (GtkWidget      *widget,
       cairo_fill (cr);
 #endif
 
-#if 0      
+#if 0
       gtk_paint_focus (widget->style, widget->window,
                        &event->area, widget, "meta-tab-image",
                        x, y, w, h);
@@ -865,13 +914,13 @@ selectable_workspace_new (MetaWorkspace *workspace)
 {
   GtkWidget *widget;
   double screen_aspect;
-  
+
   widget = g_object_new (meta_select_workspace_get_type (), NULL);
 
   screen_aspect = (double) workspace->screen->rect.height /
                   (double) workspace->screen->rect.width;
-  
-  /* account for select rect */ 
+
+  /* account for select rect */
   gtk_widget_set_size_request (widget,
                                MINI_WORKSPACE_WIDTH + SELECT_OUTLINE_WIDTH * 2,
                                MINI_WORKSPACE_WIDTH * screen_aspect + SELECT_OUTLINE_WIDTH * 2);
@@ -898,8 +947,8 @@ unselect_workspace (GtkWidget *widget)
 static void meta_select_workspace_class_init (MetaSelectWorkspaceClass *klass);
 
 #if GTK_CHECK_VERSION(3, 0, 0)
-static gboolean meta_select_workspace_draw         (GtkWidget      *widget,
-                                                    cairo_t        *cr);
+static gboolean meta_select_workspace_draw (GtkWidget *widget,
+                                            cairo_t   *cr);
 #else
 static gboolean meta_select_workspace_expose_event (GtkWidget      *widget,
                                                     GdkEventExpose *event);
@@ -925,9 +974,9 @@ meta_select_workspace_get_type (void)
         (GInstanceInitFunc) NULL,
       };
 
-      workspace_type = g_type_register_static (GTK_TYPE_DRAWING_AREA, 
-                                               "MetaSelectWorkspace", 
-                                               &workspace_info, 
+      workspace_type = g_type_register_static (GTK_TYPE_DRAWING_AREA,
+                                               "MetaSelectWorkspace",
+                                               &workspace_info,
                                                0);
     }
 
@@ -938,9 +987,9 @@ static void
 meta_select_workspace_class_init (MetaSelectWorkspaceClass *klass)
 {
   GtkWidgetClass *widget_class;
-  
+
   widget_class = GTK_WIDGET_CLASS (klass);
-  
+
   #if GTK_CHECK_VERSION(3, 0, 0)
   widget_class->draw = meta_select_workspace_draw;
   #else
@@ -959,7 +1008,7 @@ meta_convert_meta_to_wnck (MetaWindow *window, MetaScreen *screen)
   WnckWindowDisplayInfo wnck_window;
   wnck_window.icon = window->icon;
   wnck_window.mini_icon = window->mini_icon;
-  
+
   wnck_window.is_active = FALSE;
   if (window == window->display->expected_focus_window)
     wnck_window.is_active = TRUE;
@@ -994,7 +1043,7 @@ meta_select_workspace_draw (GtkWidget *widget,
   GList *tmp, *list;
 
   workspace = META_SELECT_WORKSPACE (widget)->workspace;
-              
+
   list = meta_stack_list_windows (workspace->screen->stack, workspace);
   n_windows = g_list_length (list);
   windows = g_new (WnckWindowDisplayInfo, n_windows);
@@ -1011,7 +1060,7 @@ meta_select_workspace_draw (GtkWidget *widget,
       ignoreable_sticky = window->on_all_workspaces &&
                           workspace != workspace->screen->active_workspace;
 
-      if (window->skip_pager || 
+      if (window->skip_pager ||
           !meta_window_showing_on_its_workspace (window) ||
           window->unmaps_pending ||
           ignoreable_sticky)
@@ -1044,7 +1093,7 @@ meta_select_workspace_draw (GtkWidget *widget,
                        n_windows);
 
   g_free (windows);
-  
+
   if (META_SELECT_WORKSPACE (widget)->selected)
     {
       GtkStyleContext *context;
@@ -1083,11 +1132,11 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
   int i, n_windows;
   GList *tmp, *list;
   GtkAllocation allocation;
-  
+
   gtk_widget_get_allocation(widget, &allocation);
 
   workspace = META_SELECT_WORKSPACE (widget)->workspace;
-              
+
   list = meta_stack_list_windows (workspace->screen->stack, workspace);
   n_windows = g_list_length (list);
   windows = g_new (WnckWindowDisplayInfo, n_windows);
@@ -1104,7 +1153,7 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
       ignoreable_sticky = window->on_all_workspaces &&
                           workspace != workspace->screen->active_workspace;
 
-      if (window->skip_pager || 
+      if (window->skip_pager ||
           !meta_window_showing_on_its_workspace (window) ||
           window->unmaps_pending ||
           ignoreable_sticky)
@@ -1135,7 +1184,7 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
                        n_windows);
 
   g_free (windows);
-  
+
   if (META_SELECT_WORKSPACE (widget)->selected)
     {
       style = gtk_widget_get_style (widget);
@@ -1157,4 +1206,4 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
   return TRUE;
 }
 
-#endif
\ No newline at end of file
+#endif
