@@ -1,8 +1,8 @@
 /* WM tester main() */
 
-/* 
+/*
  * Copyright (C) 2001 Havoc Pennington
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
@@ -12,7 +12,7 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
@@ -43,17 +43,17 @@ main (int argc, char **argv)
   int i;
   gboolean do_evil;
   gboolean do_icon_windows;
-  
-  gtk_init (&argc, &argv);  
-  
+
+  gtk_init (&argc, &argv);
+
   do_evil = FALSE;
   do_icon_windows = FALSE;
-  
+
   i = 1;
   while (i < argc)
     {
       const char *arg = argv[i];
-      
+
       if (strcmp (arg, "--help") == 0 ||
           strcmp (arg, "-h") == 0 ||
           strcmp (arg, "-?") == 0)
@@ -64,20 +64,20 @@ main (int argc, char **argv)
         do_icon_windows = TRUE;
       else
         usage ();
-      
+
       ++i;
     }
 
   /* Be sure some option was provided */
   if (! (do_evil || do_icon_windows))
     return 1;
-  
+
   if (do_evil)
     set_up_the_evil ();
 
   if (do_icon_windows)
     set_up_icon_windows ();
-  
+
   gtk_main ();
 
   return 0;
@@ -93,9 +93,9 @@ evil_timeout (gpointer data)
   int len;
   int create_count;
   int destroy_count;
-  
-  len = g_slist_length (evil_windows);  
-  
+
+  len = g_slist_length (evil_windows);
+
   if (len > 35)
     {
       create_count = 2;
@@ -109,7 +109,7 @@ evil_timeout (gpointer data)
 
   /* Create some windows */
   n_windows = g_random_int_range (0, create_count);
-  
+
   i = 0;
   while (i < n_windows)
     {
@@ -117,21 +117,25 @@ evil_timeout (gpointer data)
       GtkWidget *c;
       int t;
       GtkWidget *parent;
-      
+
       w = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
-      #if GTK_CHECK_VERSION(3, 0, 0)
-      		#define gtk_widget_set_uposition gtk_window_move
-      #endif
-
+#if GTK_CHECK_VERSION (3, 0, 0)
+      gtk_window_move (GTK_WINDOW (w),
+                       g_random_int_range (0,
+                                           gdk_screen_width ()),
+                       g_random_int_range (0,
+                                           gdk_screen_height ()));
+#else
       gtk_widget_set_uposition (w,
                                 g_random_int_range (0,
                                                     gdk_screen_width ()),
                                 g_random_int_range (0,
                                                     gdk_screen_height ()));
+#endif
 
       parent = NULL;
-      
+
       /* set transient for random window (may create all kinds of weird cycles) */
       if (len > 0)
         {
@@ -139,22 +143,22 @@ evil_timeout (gpointer data)
           if (t >= 0)
             {
               parent = g_slist_nth_data (evil_windows, t);
-              
+
               if (parent != NULL)
                 gtk_window_set_transient_for (GTK_WINDOW (w), GTK_WINDOW (parent));
             }
         }
-      
+
       if (parent != NULL)
         c = gtk_button_new_with_label ("Evil Transient!");
       else
         c = gtk_button_new_with_label ("Evil Window!");
       gtk_container_add (GTK_CONTAINER (w), c);
-      
+
       gtk_widget_show_all (w);
-      
+
       evil_windows = g_slist_prepend (evil_windows, w);
-      
+
       ++i;
     }
 
@@ -166,7 +170,7 @@ evil_timeout (gpointer data)
       while (i < n_windows)
         {
           GtkWidget *w;
-          
+
           w = g_slist_nth_data (evil_windows,
                                 g_random_int_range (0, len));
           if (w)
@@ -175,11 +179,11 @@ evil_timeout (gpointer data)
               evil_windows = g_slist_remove (evil_windows, w);
               gtk_widget_destroy (w);
             }
-          
+
           ++i;
         }
     }
-  
+
   return TRUE;
 }
 
@@ -197,7 +201,7 @@ set_up_icon_windows (void)
 
   /* Create some windows */
   n_windows = 9;
-  
+
   i = 0;
   while (i < n_windows)
     {
@@ -205,35 +209,53 @@ set_up_icon_windows (void)
       GtkWidget *c;
       GList *icons;
       GdkPixbuf *pix;
-      
+#if GTK_CHECK_VERSION (3, 0, 0)
+      int size  = 0;
+#endif
+
       w = gtk_window_new (GTK_WINDOW_TOPLEVEL);
       c = gtk_button_new_with_label ("Icon window");
       gtk_container_add (GTK_CONTAINER (w), c);
 
       icons = NULL;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+      gtk_icon_size_lookup (GTK_ICON_SIZE_LARGE_TOOLBAR, NULL, &size);
+      pix = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "gtk-save", size, 0, NULL);
+#else
       pix = gtk_widget_render_icon (w,
                                     GTK_STOCK_SAVE,
                                     GTK_ICON_SIZE_LARGE_TOOLBAR,
                                     NULL);
-      
+#endif
+
       icons = g_list_append (icons, pix);
 
       if (i % 2)
         {
+#if GTK_CHECK_VERSION (3, 0, 0)
+          gtk_icon_size_lookup (GTK_ICON_SIZE_DIALOG, NULL, &size);
+          pix = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "gtk-save", size, 0, NULL);
+#else
           pix = gtk_widget_render_icon (w,
                                         GTK_STOCK_SAVE,
                                         GTK_ICON_SIZE_DIALOG,
                                         NULL);
+#endif
           icons = g_list_append (icons, pix);
         }
 
       if (i % 3)
         {
+#if GTK_CHECK_VERSION (3, 0, 0)
+          gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, NULL, &size);
+          pix = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), "gtk-save", size, 0, NULL);
+#else
           pix = gtk_widget_render_icon (w,
                                         GTK_STOCK_SAVE,
                                         GTK_ICON_SIZE_MENU,
                                         NULL);
+#endif
           icons = g_list_append (icons, pix);
         }
 
@@ -241,9 +263,9 @@ set_up_icon_windows (void)
 
       g_list_foreach (icons, (GFunc) g_object_unref, NULL);
       g_list_free (icons);
-      
+
       gtk_widget_show_all (w);
-      
+
       ++i;
     }
 }
